"""
 * BluetoothPanel.py - SettingsGUI - Bluetooth Settings
 *
 * (C) 2007 by Kristian Mueller <kristian-m@kristian-m.de>
 * All Rights Reserved
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
"""

import sys
import os
import time
import gtk
import SysFSAccess
from GlobalConfiguration import *
from SysFSAccess import *
from ProcessInterface import *
#from settingsgui.GlobalConfiguration import *
#from settingsgui.SysFSAccess import *
#from settingsgui.ProcessInterface import *

class BluetoothPanel(gtk.VBox):
    """
    Settings for the Bluetooth Module of GTA01
    """
    def __init__(self):
        gtk.VBox.__init__(self, False, 0)
        self.address = ""
        self.create_notebook_page()
    
    def create_notebook_page(self):
        self.set_border_width(0)

        ## Power State of Bluetooth Module
        cell_frame = gtk.Frame("Bluetooth State")
        upper_box = gtk.HBox()
        upper_box.set_border_width(15)

        # power 
        self.power_state_cbtn = gtk.CheckButton("Powered")
        self.power_state_cbtn.set_sensitive(1)
        self.power_state_cbtn.set_active(self.get_power_state())
        self.power_state_cbtn.connect('pressed', \
                        lambda *w: self.power_state_cbtn.set_inconsistent(1))
        self.power_state_cbtn.connect('released', self.toggle_power)
        upper_box.pack_start(self.power_state_cbtn, True, True, 0)

        # discoverability
        self.visible_state_cbtn = gtk.CheckButton("Visible")
        self.visible_state_cbtn.set_sensitive(1)
        self.visible_state_cbtn.set_active(self.get_power_state())
        self.visible_state_cbtn.connect('pressed', \
                        lambda *w: self.visible_state_cbtn.set_inconsistent(1))
        self.visible_state_cbtn.connect('released', self.toggle_power)
        upper_box.pack_start(self.visible_state_cbtn, True, True, 0)

        # pand
        self.pand_state_cbtn = gtk.CheckButton("allow PAN")
        self.pand_state_cbtn.set_sensitive(1)
        self.pand_state_cbtn.connect('pressed', \
                        lambda *w: self.pand_state_cbtn.set_inconsistent(1))
        self.pand_state_cbtn.connect('released', self.toggle_listen_pand)
        upper_box.pack_start(self.pand_state_cbtn, True, True, 0)

        cell_frame.add(upper_box)
        self.pack_start(cell_frame, False, False, 0)

        ## Info on BT state
        info_frame = gtk.Frame("Bluetooth Informations")
        info_box = gtk.HBox()
        info_box.set_border_width(15)
        self.address_label = gtk.Label("Address: %s" % self.get_address())
        info_box.add(self.address_label)
        info_frame.add(info_box)
        self.pack_start(info_frame, False, True, 1)


        scan_frame = gtk.Frame("Devices in range")
        scan_frame_box = gtk.VBox()
        scan_frame_box.set_border_width(15)
        (scroll_win, self.tree_view1, self.list_store1) = \
                            self.make_list_view(2, ["Name", "Address"])
        scan_frame_box.pack_start(scroll_win, True, True, 0)

        scan_btn_box = gtk.HBox()
        update_btn = gtk.Button("Update\nList")
        update_btn.connect('clicked', self.update_list)
        scan_btn_box.add(update_btn)
        connect_btn = gtk.Button("Connect\n(PAN)")
        connect_btn.connect('clicked', self.connect_to_peer)
        scan_btn_box.add(connect_btn)
        scan_frame_box.pack_start(scan_btn_box, False, True, 0)
        scan_frame.add(scan_frame_box)

        self.pack_start(scan_frame, True, True, 0)

        ## finish notebook page creation
        self.show_all()

################################################################################
######################### build GUI helper #####################################
################################################################################
    def make_list_view(self, number, names):
        scroll_win = gtk.ScrolledWindow()
        scroll_win.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)

        list_store = gtk.ListStore(*(str,) * number)

        tree_view = gtk.TreeView(list_store)
        cell = gtk.CellRendererText()
        cell.set_property('editable', True)

        tvcolumn = []
        for i in range(number):
            tvcolumn.append(gtk.TreeViewColumn(names[i], cell, markup=i))
            tree_view.append_column(tvcolumn[i])
            tvcolumn[i].set_sort_column_id(i)
            tvcolumn[i].set_resizable(True)
        
        scroll_win.add(tree_view)
        return (scroll_win, tree_view, list_store)

################################################################################
####################### Callbacks from GUI #####################################
################################################################################
    def toggle_power(self, event):
        if not self.get_power_state():
            set_sysfs_value(SYSFS_ENTRY_BLUETOOTH_POWER, 1)
        else:
            set_sysfs_value(SYSFS_ENTRY_BLUETOOTH_POWER, 0)
        
        if self.power_state_cbtn.get_active() != self.get_power_state():
            self.power_state_cbtn.get_active()

        ## have to wait for async init :-( - TODO - make updates async.
        time.sleep(2)
        self.update_infos()
        self.update_list(None)
        self.power_state_cbtn.set_inconsistent(0)

#    #def abl()

    def toggle_listen_pand(self, event):
        if self.pand_state_cbtn.get_active():
            print "Starting pand [pand -s]"
            os.system("pand -s")
        else:
            print "pand already listening for connections"
            self.pand_state_cbtn.set_active(1)
        self.pand_state_cbtn.set_inconsistent(0)

    def get_power_state(self):
        state = get_sysfs_value(SYSFS_ENTRY_BLUETOOTH_POWER)
        state = state.strip()
        if state.isdigit():
            return int(state)
        else:
            return 0

    def connect_to_peer(self, event):
        ## get selected entry
        if len(self.address) <= 0:
            return
        (model, model_iter) = self.tree_view1.get_selection().get_selected()
        if model_iter >= 1:
            name = model.get_value(model_iter, 0)  # column is first (name)
            addr = model.get_value(model_iter, 1)  # column is second (adr)
            os.system("pand -c %s" % addr)
            time.sleep(2)
            ## convert last number of addr to decimal and user as last ip number
            os.system("ip a add 10.0.0.%s/24 dev bnep0" % \
                                    int(self.address.split(":")[-1], 16))
            ## set main ip of bnep0 (in case a different ip was set)
            os.system("ifconfig bnep0 10.0.0.%s/24 dev bnep0" % \
                                    int(self.address.split(":")[-1], 16))
            ## set 10.0.0.1 to default GW - won't work if GW is set already
            os.system("ip r add default via 10.0.0.1")

    def update_list(self, event):
        self.hcitool = ProcessInterface("%s scan" % HCITOOL_CMD)
        self.list_store1.clear()
        self.hcitool.register_event_handler(":", self.add_scan_list_entry)

    def add_scan_list_entry(self, string):
        if string.find(":") >= 0:
            print(string)
            self.list_store1.append([string.strip().split("\t")[1], \
                                            string.strip().split("\t")[0]])



################################################################################
####################### Interface to hciconfig #################################
################################################################################
    def update_infos(self):
        self.address_label.set_text("Address: %s" % self.get_address())

    def get_address(self):
        hciconfig = ProcessInterface("%s %s" % (HCICONFIG_CMD, BLUETOOTH_DEVICE))
        output = hciconfig.read_from_process()
        # hciconfig.close_process() ## has no effect anyway :-( TODO - fix
        if output.find("BD Address: "):
            if len(output.split("BD Address: ")) > 1 and \
                    len(output.split("BD Address: ")[1].split(" ")) > 1:
                self.address = output.split("BD Address: ")[1].split(" ")[0]
                return output.split("BD Address: ")[1].split(" ")[0]
            else:
                return "offline"
        return "unknown"

    def get_features(self):
        """using the hciconfig tool to get the list of supported features"""
        
